function drpBook(bookData) {
	this._bookData = bookData;
	this._html = null;
	this._zoomEnabled = false;
	this._zoomedLevel = 0;
	this._zoomDragBinded = false;
	this._fullscreenEnabled = false;
	this._isDragging = false;
	this._zoomTargetEls = [];
	this._zoomPrevEvent = null;
	// To get page iterator by it's ID
	this._pageIdToIter = {};
	this._pageIdToIterWasSet = false;
	this._audioPagingObj = null;
	this._pagesCount = 0;
	this._promoLink = null;
	this._foundPages = [];
	this._currentFoundOpenPage = 0;
	this._searchValues = {};
	this._anim = 300;
	this._orStageSize = {width: 0, height: 0};	// or == Original
	this._orBookSize = {width: 0, height: 0};
	this._orShellSize = {width: 0, height: 0};
	this._orToolbarSize = {width: 0, height: 0};
	this._stageSizeChanged = false;
	this._fakePages = [];
	this._zomScrollK = 3;
	this._baseHtml = '';
	this._zoomOutInitPos = {top: 0, left: 0};
	this._promoLinkInitPos = {top: 0, left: 0};
	this._maxZoomScroll = {top: 0, left: 0};
	this._toolbarInResponsive = false;
	this._init();
}
drpBook.prototype._init = function(reinit) {
	var self = this
	,	stage = this.getStage();

	this._pagesCount = this._bookData.pages.length;
	if(parseInt(this._bookData.params['en_audio_paging'])
		&& this._bookData.params['audio_paging'] 
		&& this._bookData.params['audio_paging'] != ''
		&& typeof(Audio) !== 'undefined'
	) {
		this._audioPagingObj = new Audio( this._bookData.params['audio_paging'] );
	}
	if(!this._bookData.pages) {
		// !!!DO SOMETHING HERE!!!
		return false;
	}
	if(this._isHardcover()) {
		// First and second page - should be with hardcover
		this._bookData.pages[0].hardcover = true;
		this._bookData.pages[1].hardcover = true;
		// Last page - with hardcover
		this._bookData.pages[ this._bookData.pages.length-1 ].hardcover = true;
		// If number of pages - is even number - then pre-last page should be hardcover too
		if(this._bookData.pages.length % 2 === 0) {
			this._bookData.pages[ this._bookData.pages.length-2 ].hardcover = true;
		}
	}
	this._baseHtml = stage.parent().html();
	this._bookData.params['gradients'] = parseInt(this._bookData.params['gradients']);
	this._bookData.params['show_border_shadow'] = parseInt(this._bookData.params['show_border_shadow']);
	this._html = jQuery('#book_'+ this._bookData.term_id).turn({
		gradients: this._bookData.params['gradients']
	,	acceleration: true
	,	autoCenter: true
	,	width: this._bookData.params['width']
	,	height: this._bookData.params['height']
	,	pages: this._pagesCount
	,	direction: this._bookData.params['direction']
	,	duration: this._bookData.params['duration']
	,	when: {
			turning: function(e, page, openedPageNums) {
				self._setDisplayPagesNum(openedPageNums);
				if(self._audioPagingObj) {
					// Stop audio in case it didn't finish prev. play
					if(self._audioPagingObj.currentTime) {
						self._audioPagingObj.pause();
						self._audioPagingObj.currentTime = 0;
					}
					// Play sound again
					self._audioPagingObj.play();
				}
				self._removeFakePages();
				self._removeBorderShadow();
			}
		,	turned: function(e, page, openedPageNums) {
				self._setDisplayPagesNum(openedPageNums);
				
				var displayPageNums = self._getDisplayPagesNum()
				,	toolbar = self.getToolbar();
				
				self.set('displayPageNums', displayPageNums);
				toolbar.find('.dprTbPageNumTxt').val( displayPageNums.join(' - ')+ ' / '+ self._pagesCount );
				self._processHightlightSearch( page );
				self._checkBorderShadow();
				if(self._zoomEnabled) {
					self._adjustPosForZommed();
					/*self.getHtml().css({
						'margin': '0'
					});*/
					/*setTimeout(function(){
						self.getHtml().css({marginLeft: 0});
					}, 100);*/
				} else {
					self._createFakePages();
				}
			}
		,	missing: function(event, pages) {
				for(var i = 0; i < pages.length; i++) {
					self._addPage(pages[i]);
					// Should investigate here
					/*if(parseInt(self._bookData.params.enable_pages_dynamic_preload) && i > 1)	// Load current and 2 other pages, and then - stop
						break;*/
				}
			}
		}
	});
	if(!reinit) {
		jQuery(document).keypress(function(e){
			if(e.key == 'Esc' || e.keyCode == 27) {
				if(self._zoomEnabled)
					self.zoomOut();
				if(self._fullscreenEnabled)
					self._fullscreenOff();
			}
		});
		this._openPageOnLoad();
		this._bookData.params['fake_pages'] = parseInt(this._bookData.params['fake_pages']);
	}
	this._createFakePages();
	this._checkBorderShadow();
};
drpBook.prototype.getPagesCount = function() {
	return this._pagesCount;
};
drpBook.prototype._returnPromoLinkLocation = function() {
	this._prInit();
	if(this._promoLink) {
		this._promoLink.css({
			'position': 'absolute'
		,	'top': 0
		,	'right': 0
		});
	}
};
drpBook.prototype._prInit = function() {
	this._promoLink = this.getStage().children('a[target="_blank"]');
	if(this._promoLink.size() == 0)
		this._promoLink = false;
};
drpBook.prototype.setPlLocation = function() {
	if(!this._zoomEnabled) return;
	this._prInit();
	if(this._promoLink) {
		if(this._fullscreenEnabled) {
			this._promoLink.css({
				'position': 'fixed'
			,	'right': 0
			,	'top': 0
			});
			return;
		}
		var	promoLinkPos = {top: 0, left: 0}
		,	docScrollTop = jQuery(document).scrollTop()
		,	docScrollLeft = jQuery(document).scrollLeft();
		if(!this._promoLinkInitPos.top) {
			var stage = this.getStage()
			,	stageOffset = stage.offset()
			,	promoWidth = this._promoLink.width() + 10;
			this._promoLinkInitPos.top = stageOffset.top;
			this._promoLinkInitPos.left = stageOffset.left + stage.width() - promoWidth - 35;
			this._promoLinkInitPos.width = promoWidth;
		}
		promoLinkPos.top = this._promoLinkInitPos.top - docScrollTop;
		promoLinkPos.left = this._promoLinkInitPos.left - docScrollLeft;
		this._promoLink.css({
			'position': 'fixed'
		,	'left': promoLinkPos.left
		,	'top': promoLinkPos.top
		,	'width': this._promoLinkInitPos.width+ 'px'
		});
	}
};
drpBook.prototype._restorePromoLinkLocation = function() {
	if(this._promoLink) {
		this._promoLink.css({
			'top': 0
		,	'right': 0
		});
	}
};
drpBook.prototype._isHardcover = function() {
	return (this._bookData.params['cover_type'] == 'hardcover');
};
drpBook.prototype._openPageOnLoad = function() {
	var hashData = toeGetHashParams()
	,	openPageNumber = 0;	// Page number, that should be opened, from 1 to N
	if(hashData && hashData.length) {
		for(var i = 0; i < hashData.length; i++) {
			if(typeof(hashData[i]) !== 'string') continue;
			// It should be in format page123, where 123 - is page ID
			if(strpos(hashData[i], 'page') === 0) {
				var openPageId = parseInt(str_replace(hashData[i], 'page', ''));
				if(openPageId) {
					var pageIter = this._getPageIterById(openPageId);
					if(pageIter !== false) {
						openPageNumber = pageIter + 1;	// In turn we count from 1
						break;
					}
				}
			}
		}
	}
	if(!openPageNumber && this._bookData.params['open_on_page']) {
		openPageNumber = parseInt( this._bookData.params['open_on_page'] );
	}
	if(openPageNumber) {
		this.getHtml().turn('page', openPageNumber);
	}
};
drpBook.prototype._getPageIterById = function(pageId) {
	if(!this._pageIdToIterWasSet) {
		for(var i in this._bookData.pages) {
			this._pageIdToIter[ this._bookData.pages[i].ID ] = parseInt(i);
		}
		this._pageIdToIterWasSet = true;
	}
	return (typeof(this._pageIdToIter[pageId]) === 'undefined' ? false : this._pageIdToIter[pageId]);
};
drpBook.prototype._addPage = function(page) {
	var html = jQuery('#book_'+ this._bookData.term_id);
	
	// Create a new element for this page
	var element = jQuery('<div />', {});
	if(this._bookData.pages[page - 1].hardcover) {
		element.addClass('hard');
	}
	
	if(this._bookData.params['gradients']) {
		//element.addClass('gradients');
	}
	// Add the page to the flipbook
	if (html.turn('addPage', element, page)) {
		// Add the initial HTML
		// It will contain a loader indicator
		element.html('<div class="loader"></div><div class="page-main-content"></div>');
		// Load the page
		this._loadPage(page, element);
	}
	this._bookData.pages[page - 1].html_element = element;
};
drpBook.prototype._loadPage = function(page, element) {
	element.find('.page-main-content').append( this._bookData.pages[page-1].post_content );
	var images = element.find('img')
	,	imgCount = images.size();
	// Hide loader after all images will be loaded
	if(imgCount) {
		images.load(function(){
			imgCount--;
			if(imgCount <= 0) {
				element.find('.loader').hide();
			}
		});
	} else {
		element.find('.loader').hide();
	}
	if(this._bookData.params['gradients']) {
		// If page - is an image - then we will place gradient element uppon it
		if(parseInt(this._bookData.pages[page-1].attach_id)) {
			var gradient = jQuery('<div class="gradients separateGradient" />');
			element.hasClass('even') ? gradient.addClass('even') : gradient.addClass('odd');
			gradient.addClass(this._bookData.params['direction'])
			element.append(gradient);
			element.addClass('dprAttachPage');
		} else {	// Else - not allowed to place gradient element to page as all it's html content will not be clickable
			element.addClass('gradients').addClass(this._bookData.params['direction']);
		}
	}
};
drpBook.prototype._setDisplayPagesNum = function(openedPageNums) {
	var displayPageNums = [];
	for(var i in openedPageNums) {
		if(parseInt(openedPageNums[i])) {
			displayPageNums.push( openedPageNums[i] );
		}
	}
	this.set('displayPageNums', displayPageNums);
};
drpBook.prototype._getDisplayPagesNum = function() {
	return this.get('displayPageNums');
}
drpBook.prototype.getHtml = function() {
	return this._html;
};
drpBook.prototype.getData = function() {
	return this._bookData;
};
drpBook.prototype.getStage = function() {
	return jQuery('#book_stage_'+ this._bookData.term_id);
};
drpBook.prototype.getToolbar = function() {
	return jQuery('#book_toolbar_'+ this._bookData.term_id);
};
drpBook.prototype.getBookShell = function() {
	return jQuery('#book_shell_'+ this._bookData.term_id);
};
drpBook.prototype.getFullscreenBtn = function() {
	return this.getToolbar().find('.dprTbBtnFullscreen');
};
drpBook.prototype.getSearchForm = function() {
	return this.getToolbar().find('.dprTbSearchForm');
};
drpBook.prototype.getSearchFormShell = function() {
	return this.getToolbar().find('.dprTbSearchShell');
};
drpBook.prototype.getPrevSearchBtn = function() {
	return this.getSearchFormShell().find('.dprTbBtnPrevSearch');
};
drpBook.prototype.getNextSearchBtn = function() {
	return this.getSearchFormShell().find('.dprTbBtnNextSearch');
};
/**
 * Should be all, except zoom out btn
 */
drpBook.prototype.getToolbarMain = function() {
	return this.getToolbar().find('.btnTbMain');
};
drpBook.prototype.getZoomOutBtn = function() {
	return this.getStage().find('.dprTbZoomOutShell');
};
drpBook.prototype.set = function(key, val) {
	this._bookData[ key ] = val;
};
drpBook.prototype.get = function(key) {
	return this._bookData[ key ];
};
drpBook.prototype.isset = function(key) {
	typeof(this._bookData[ key ]) === 'undefined' ? false : true;
};
drpBook.prototype.setZoomOutPos = function(stage) {
	if(!this._zoomEnabled) return;
	var	zoomOut = this.getZoomOutBtn()
	,	incPos = 30;
	if(this._fullscreenEnabled) {
		zoomOut.css({
			'left': incPos
		,	'bottom': zoomOut.height() + incPos
		,	'position': 'fixed'
		});
		return;
	}
	var docScrollTop = jQuery(document).scrollTop()
	,	docScrollLeft = jQuery(document).scrollLeft()
	,	zoomOutPos = {top: 0, left: 0};
	if(!this._zoomOutInitPos.top) {
		var stage = stage ? stage : this.getStage()
		,	stageOffset = stage.offset()
		,	stageHeight = stage.height();
		this._zoomOutInitPos.top = stageOffset.top + stageHeight - incPos - zoomOut.height();
		this._zoomOutInitPos.left = stageOffset.left + incPos;
	}
	zoomOutPos.top = this._zoomOutInitPos.top - docScrollTop;
	zoomOutPos.left = this._zoomOutInitPos.left - docScrollLeft;
	zoomOut.css({
		'left': zoomOutPos.left
	,	'top': zoomOutPos.top
	,	'position': 'fixed'
	});
};
/**
 * Zoom book
 * @param {int} zoomLevel Zoom level
 */
drpBook.prototype.zoomIn = function(zoomLevel) {
	this._zoomEnabled = true;
	zoomLevel = zoomLevel ? zoomLevel : this._bookData.params['zoom_level'];
	zoomLevel = parseInt(zoomLevel);
	this._zoomedLevel = zoomLevel;
	var bookHtml = this.getHtml()
	,	bookStage = this.getStage()
	,	bookToolbarMain = this.getToolbarMain()
	,	bookWidth = bookHtml.width()
	,	zoomOutBtn = this.getZoomOutBtn()
	,	toolbar = this.getToolbar()
	,	prevToolbarStyleAttr = toolbar.attr('style');

	if(!prevToolbarStyleAttr)
		prevToolbarStyleAttr = '';
	
	//console.log(bookHtml.height(), bookStage.height(), bookHtml.width(), bookStage.width());
	this._maxZoomScroll.top = bookHtml.height() * this._zoomedLevel - bookStage.height();
	this._maxZoomScroll.left = bookHtml.width() * this._zoomedLevel - bookStage.width();
	if(this._maxZoomScroll.top >= 0)
		this._maxZoomScroll.top += 20;	// For scrollbar
	if(this._maxZoomScroll.left >= 0)
		this._maxZoomScroll.left += 20;	// For scrollbar
	
	this.set('prevZoom', (this.isset('prevZoom') ? this.get('prevZoom') : 1));
	this.set('prevTop', (this.isset('prevTop') ? this.get('prevTop') : bookHtml.css('top')));
	this.set('prevLeft', (this.isset('prevLeft') ? this.get('prevLeft') : bookHtml.css('left')));
	this.set('prevWidth', (this.isset('prevWidth') ? this.get('prevWidth') : bookWidth));
	this.set('prevStageOverflow', (this.isset('prevStageOverflow') ? this.get('prevStageOverflow') : bookStage.css('overflow')));
	this.set('prevStyleAttr', (this.isset('prevStyleAttr') ? this.get('prevStyleAttr') : bookHtml.attr('style')));
	this.set('prevToolbarStyleAttr', (this.isset('prevToolbarStyleAttr') ? this.get('prevToolbarStyleAttr') : prevToolbarStyleAttr));
	
	bookHtml.zoom( zoomLevel ).css({
		'top': '0'
	,	'margin': '0'
	});
	toolbar.css({
		'position': 'absolute'
	,	'top': '0'
	,	'left': '0'
	,	'width': '100%'
	});
	this._adjustPosForZommed();
	bookStage.css('overflow', 'scroll');
	bookToolbarMain.hide();
	zoomOutBtn.show();
	this._zoomMakeDraggable();
	this._imgSetZoomSizes();
	this._removeFakePages();
	this.toggleSearchForm(false);
	this.setZoomOutPos();
	this.setPlLocation();
};
drpBook.prototype._imgSetZoomSizes = function(restore) {
	var bookHtml = this.getHtml()
	,	pageNums = this._getDisplayPagesNum()	// we need all display page numbers
	,	turnData = bookHtml.turn('data')
	,	self = this
	,	savedImgs = {};
	for(var i in pageNums) {
		var pageNum = pageNums[i];
		if(turnData && turnData.pageObjs && turnData.pageObjs[ pageNum ]) {
			var page = turnData.pageObjs[ pageNum ]
			,	imgs = page.find('img');
			if(imgs && imgs.size()) {
				imgs.each(function(){
					var attachId = toeGetImgAttachId(this)
					,	currentImg = this;
					if(attachId 
						&& self._bookData.pages[ pageNum-1 ].attach_sizes 
						&& self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ]
						&& self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].full
						&& !self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].not_transform
					) {
						if(restore) {	// Restore small sizes
							if(self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page) {
								savedImgs[ attachId ] = currentImg;
								jQuery('<img />')
								.attr('src', self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page)
								.attr('data-attach-id', attachId)
								.attr('data-page-num', pageNum)
								.load(function(){
									var _attachId = jQuery(this).data('attach-id')
									,	_pageNum = jQuery(this).data('page-num');
									jQuery(currentImg).attr('src', self._bookData.pages[ _pageNum-1 ].attach_sizes[ _attachId ].on_page);
									jQuery(this).remove();
								});
							}
						} else {	// Set full size for image
							var currentSrc = jQuery(this).attr('src');
							self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page = currentSrc;
							// Image already are in full size
							if(self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].on_page == self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].full) {
								self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].not_transform = true;
							} else {
								// Create temp img - to not change src of current image until new src will be loaded
								savedImgs[ attachId ] = currentImg;
								jQuery('<img />')
								.attr('src', self._bookData.pages[ pageNum-1 ].attach_sizes[ attachId ].full)
								.attr('data-attach-id', attachId)
								.attr('data-page-num', pageNum)
								.load(function(){
									var _attachId = jQuery(this).data('attach-id')
									,	_pageNum = jQuery(this).data('page-num');
									jQuery( savedImgs[ _attachId ] ).attr('src', self._bookData.pages[ _pageNum-1 ].attach_sizes[ _attachId ].full);
									jQuery(this).remove();
								});
							}
						}
					}
				});
			}
		}
	}
};
drpBook.prototype._imgRestoreZoomSizes = function() {
	this._imgSetZoomSizes(true);
};
/**
 * Return book to normal zoom
 */
drpBook.prototype.zoomOut = function() {
	this._zoomEnabled = false;
	var bookHtml = this.getHtml()
	,	bookStage = this.getStage()
	,	bookToolbarMain = this.getToolbarMain()
	,	zoomOutBtn = this.getZoomOutBtn();
	
	/*bookHtml.zoom( this.get('prevZoom') ).css({
		'top': this.get('prevTop')
	,	'left': this.get('prevLeft')
	,	'width': this.get('prevWidth')
	,	'margin': '0 auto'
	,	'transform-origin': ''
	,	'transform': ''
	});*/
	/*bookHtml.attr('style', this.get('prevStyleAttr')).css({
		'margin': '0 auto'
	});*/
	//toolbar.attr('style', this.get('prevToolbarStyleAttr'));
	//bookStage.css('overflow', this.get('prevStageOverflow'));
	bookToolbarMain.show();
	zoomOutBtn.hide();
	this.center();
	this._zoomMakeUndraggable();
	this._imgRestoreZoomSizes();
	
	var currentPage = bookHtml.turn('page')
	,	fullScreenEnabled = this._fullscreenEnabled
	,	bookStageParent = bookStage.parent();
	
	bookHtml.turn('destroy');
	bookStageParent.hide();
	bookStageParent.html( this._baseHtml );
	this._init(true);
	bookHtml = this.getHtml();
	bookStageParent.show();
	bookHtml.turn('page', currentPage);
	if(fullScreenEnabled)	// Fullscreen was enabled before re-init
		this._fullscreenOn();
	bookHtml.turn('center');
	
	this._restorePromoLinkLocation();
	this._createFakePages();
	this._returnPromoLinkLocation();
};
drpBook.prototype.center = function() {
	this.getHtml().turn('update').turn('center');
};
drpBook.prototype._zoomBookHtmlMouseDownCb = function(e) {
	var book = e.data.book;
	e.preventDefault ? e.preventDefault() : e.returnValue = false;
	if(book)
		book._isDragging = true;
};
drpBook.prototype._zoomDocMouseUpCb = function(e) {
	var book = e.data.book;
	if(book && book._isDragging) {
		book._isDragging = false;
		if(book._zoomTargetEls && book._zoomTargetEls.length) {
			for(var i in book._zoomTargetEls) {
				if(book._zoomTargetEls[i].element) {
					book._zoomTargetEls[i].element.isInDragList = false;
					jQuery(book._zoomTargetEls[i].element).css('cursor', book._zoomTargetEls[i].cursor);
				}
			}
			book._zoomTargetEls[i] = [];
		}
	}
};
drpBook.prototype._zoomDocMouseMoveCb = function(e) {
	var book = e.data.book;
	book._isDragging = true;
	// Always drag when move mouse
	if(book && book._isDragging) {
		if(book._zoomPrevEvent) {
			var xDisplace = e.clientX - book._zoomPrevEvent.clientX
			,	yDisplace = e.clientY - book._zoomPrevEvent.clientY
			,	bookStage = book.getStage()
			,	stageScrollLeft = bookStage.scrollLeft()
			,	stageScrollTop = bookStage.scrollTop()
			,	newScrollLeft = stageScrollLeft + xDisplace * book._zomScrollK
			,	newScrollTop = stageScrollTop + yDisplace * book._zomScrollK;
			
			if(newScrollLeft < 0)
				newScrollLeft = 0;
			else if(book._maxZoomScroll.left > 0 && newScrollLeft > book._maxZoomScroll.left)
				newScrollLeft = book._maxZoomScroll.left;
			
			if(newScrollTop < 0)
				newScrollTop = 0;
			else if(book._maxZoomScroll.top > 0 && newScrollTop > book._maxZoomScroll.top) 
				newScrollTop = book._maxZoomScroll.top;

			bookStage
				.scrollLeft( newScrollLeft )
				.scrollTop( newScrollTop );
		}
		book._zoomPrevEvent = e;
		
	}
};
drpBook.prototype._zoomMakeDraggable = function() {
	jQuery(document)
		.bind('mousemove', {book: this}, this._zoomDocMouseMoveCb);
};
drpBook.prototype._zoomMakeUndraggable = function() {
	jQuery(document)
		.unbind('mousemove', this._zoomDocMouseMoveCb);
};
drpBook.prototype.fullscreenToggle = function() {
	if(this._fullscreenEnabled) {
		this._fullscreenOff();
		this._fullscreenEnabled = false;
	} else {
		this._fullscreenOn();
		this._fullscreenEnabled = true;
	}
	this.getHtml().turn('center');
	this._createFakePages();
};
drpBook.prototype._fullscreenOn = function() {
	var fsBtn = this.getFullscreenBtn()
	,	html = this.getHtml()
	,	stage = this.getStage()
	,	bookShell = this.getBookShell()
	,	toolbar = this.getToolbarMain();
	
	this.set('fsPrevTop', (this.isset('fsPrevTop') ? this.get('fsPrevTop') : html.css('top')));
	this.set('fsPrevShellHeight', (this.isset('fsPrevShellHeight') ? this.get('fsPrevShellHeight') : bookShell.css('height')));
	this.set('stagePrevStyle', (this.isset('stagePrevStyle') ? this.get('stagePrevStyle') : stage.attr('style')));
	
	stage.css({
		'position': 'fixed'
	,	'min-width': '100%'
	,	'min-height': '100%'
	,	'left': '0'
	,	'top': '0'
	,	'z-index': '9'
	});
	toolbar.css({
		'position': 'fixed'
	,	'min-width': '100%'
	,	'left': '0'
	,	'bottom': '0'
	});
	bookShell.css({
		'height': stage.height() - toolbar.height()
	});
	html.css({
		'top': (bookShell.height() - html.height()) / 2
	});
	fsBtn.addClass('dprTbBtnFullscreenEnabled');
};
drpBook.prototype._fullscreenOff = function() {
	var fsBtn = this.getFullscreenBtn()
	,	html = this.getHtml()
	,	stage = this.getStage()
	,	bookShell = this.getBookShell()
	,	toolbar = this.getToolbarMain()
	,	stagePrevStyle = this.get('stagePrevStyle');
	if(!stagePrevStyle)
		stagePrevStyle = '';
	/*stage.css({
		'position': 'relative'
	,	'min-width': 'auto'
	,	'min-height': 'auto'
	});*/
	stage.attr('style', stagePrevStyle);
	toolbar.css({
		'position': 'static'
	,	'min-width': 'auto'
	});
	bookShell.css({
		'height':  this.get('fsPrevShellHeight')
	});
	html.css({
		'top': this.get('fsPrevTop')
	});
	fsBtn.removeClass('dprTbBtnFullscreenEnabled');
	this.center();
};
drpBook.prototype.toggleSearchForm = function(newValue) {
	if(typeof(newValue) !== 'undefined') {
		newValue 
			? this.getSearchFormShell().show( this._anim ) 
			: this.getSearchFormShell().hide( this._anim );
	} else
		this.getSearchFormShell().toggle( this._anim );
};
drpBook.prototype.search = function(value) {
	var tmpDiv = document.createElement('div');
	this._foundPages = [];
	this._searchValues = {};

	if(!value || value == '') {
		this._processHightlightSearch();
		return;
	}
	var valueLowered = value.toLowerCase();
	for(var i in this._bookData.pages) {
		tmpDiv.innerHTML = this._bookData.pages[ i ].post_content;
		var pageText = tmpDiv.textContent || tmpDiv.innerText || ''
		,	pageTextLowered = pageText ? pageText.toLowerCase() : ''
		,	startFound = false;

		if(pageText && pageText != '' && (startFound = strpos(pageTextLowered, valueLowered)) !== false) {
			var pageI = parseInt(i);
			this._foundPages.push( pageI );
			if( !this._searchValues[ pageI ] )
				this._searchValues[ pageI ] = [];
			while(startFound !== false) {
				var lastFound = pageText.substr(startFound, valueLowered.length);
				if(toeInArray(this._searchValues[ pageI ], lastFound) === -1)
					this._searchValues[ pageI ].push( lastFound );
				pageTextLowered = pageTextLowered.substr(startFound + 1);
				pageText = pageText.substr(startFound + 1);
				if(pageTextLowered && pageTextLowered != '')
					startFound = strpos(pageTextLowered, valueLowered);
				else
					startFound = false;
			}
		}
	}
	if(this._foundPages.length) {
		/*if(this._foundPages.length > 1)
			nextSearchBtn.addClass('dprTbBtnSearchActive');*/
		var book = this.getHtml();
		if(book.turn('page') == (this._foundPages[0] + 1))
			this._processHightlightSearch();
		book.turn('page', this._foundPages[0] + 1);
	} else
		this._processHightlightSearch();
};
drpBook.prototype.goToPrevSearch = function() {
	if(this._foundPages.length > 1) {
		var book = this.getHtml()
		,	currentPage = parseInt(book.turn('page'));
		if(this._currentFoundOpenPage > 0) {
			this._currentFoundOpenPage--;
			var nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			if(currentPage % 2 !== 0 && currentPage - 1 === nextSearchPage && this._currentFoundOpenPage > 0) {
				this._currentFoundOpenPage--;
				nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			}
			book.turn('page', nextSearchPage);
		}
		//this.__checkCanSearchMove();
	}
};
drpBook.prototype.goToNextSearch = function() {
	if(this._foundPages.length > 1) {
		var book = this.getHtml()
		,	currentPage = parseInt(book.turn('page'));
		if(this._currentFoundOpenPage < (this._foundPages.length - 1)) {
			this._currentFoundOpenPage++;
			var nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			if(currentPage % 2 === 0 && currentPage + 1 === nextSearchPage) {
				this._currentFoundOpenPage++;
				nextSearchPage = this._foundPages[ this._currentFoundOpenPage ] + 1;
			}
			book.turn('page', nextSearchPage);
		}
		//this.__checkCanSearchMove();
	}
};
/*drpBook.prototype.__checkCanSearchMove = function() {
	var canMoveNext = false
	,	canMovePrev = false
	,	nextSearchBtn = this.getNextSearchBtn()
	,	prevSearchBtn = this.getPrevSearchBtn();
	if(this._currentFoundOpenPage < (this._foundPages.length - 1))
		canMoveNext = true;
	if(this._currentFoundOpenPage > 0)
		canMovePrev = true;
	if(canMoveNext) {
		nextSearchBtn.addClass('dprTbBtnSearchActive');
	} else {
		nextSearchBtn.removeClass('dprTbBtnSearchActive');
	}
	if(canMovePrev) {
		prevSearchBtn.addClass('dprTbBtnSearchActive');
	} else {
		prevSearchBtn.removeClass('dprTbBtnSearchActive');
	}
};*/
drpBook.prototype._processHightlightSearch = function() {
	var displayPagesNum = this._getDisplayPagesNum();
	if(displayPagesNum && displayPagesNum.length) {
		for(var i = 0; i < displayPagesNum.length; i++) {
			var pageI = displayPagesNum[i] - 1
			,	page = this._bookData.pages[ pageI ];
			this._removeHightlightFromElement(page.html_element);
			if(this._searchValues 
				&& this._searchValues[ pageI ] 
				&& this._searchValues[ pageI ].length 
				&& page 
				&& toeInArray(pageI, this._foundPages) !== -1
			) {
				var innerHTML = page.html_element.html();
				for(var sI in this._searchValues[ pageI ] ) {
					var replaceData = {};
					replaceData[ this._searchValues[ pageI ][sI] ] = '[DPR_HIGHLIGHT_START]'+ this._searchValues[ pageI ][sI]+ '[DPR_HIGHLIGHT_END]';
					innerHTML = replaceWords(innerHTML, replaceData);
					innerHTML = str_replace(innerHTML, '[DPR_HIGHLIGHT_START]', '<span class="dprSearchHightlight">');
					innerHTML = str_replace(innerHTML, '[DPR_HIGHLIGHT_END]', '</span>');
				}
				page.html_element.html( innerHTML );
			}
		}
	}
};
drpBook.prototype._removeHightlightFromElement = function(element) {
	element.find('.dprSearchHightlight').each(function(){
		var outerHtml = this.outerHTML
		,	innerSearchText = jQuery(this.outerHTML).get(0).innerHTML;
		element.html( str_replace(element.html(), outerHtml, innerSearchText) );
	});
};
/**
 * If full width of page for example is less then width of our book - le't stry to adjust book width
 * @param {int} boundWidth Width to adjust for, current window width for example
 */
drpBook.prototype.adjustWidth = function(boundWidth) {
	if(!parseInt(boundWidth)) return;
	var bookHtml = this.getHtml()
	,	bookStage = this.getStage()
	,	bookShell = this.getBookShell()
	,	toolbar = this.getToolbar()
	,	stageWidth = bookStage.width()
	,	stageHeight = bookStage.height() 
	,	shellHeight = bookShell.height()
	,	stageSizeUpdated = false
	,	newShellHeight = (boundWidth / stageWidth) * shellHeight
	,	newStageHeight = (boundWidth / stageWidth) * stageHeight
	,	toolbarHeight = toolbar.height()
	,	kWidth = 0;	// k == koeficient
	if(boundWidth < stageWidth) {
		this._setOriginalSizes(stageWidth, stageHeight, shellHeight);
	}
	if(this._orStageSize.height) {
		kWidth = (boundWidth / this._orStageSize.width);
		newStageHeight = kWidth * this._orStageSize.height + (toolbarHeight / kWidth);
		newShellHeight = kWidth * this._orShellSize.height/* + (toolbarHeight / kWidth)*/;
	}
	if(boundWidth < stageWidth) {
		bookStage.width( boundWidth ).height( newStageHeight );
		bookShell.height( newShellHeight );
		stageSizeUpdated = true;
		this._stageSizeChanged = true;
	} else if(this._orStageSize.width && this._stageSizeChanged) {
		if(this._orStageSize.width > boundWidth) {
			bookStage.width( boundWidth ).height( newStageHeight );
			bookShell.height( newShellHeight );
		} else {
			bookStage.width( this._orStageSize.width ).height( this._orStageSize.height );
			bookShell.height( this._orShellSize.height );
			this._orStageSize.width = 0;
			this._orStageSize.height = 0;
		}
		//this._stageSizeChanged = false;
		stageSizeUpdated = true;
	}
	//alert(stageWidth+ ','+ boundWidth);

	//if(stageWidthUpdated) {
		var bookWidth = bookHtml.width()
		,	bookSizeUpdated = false
		,	adjustByWidth = boundWidth;
		if(adjustByWidth > stageWidth) {
			adjustByWidth = stageWidth;
		}
		if(adjustByWidth < bookWidth) {
			if(!this._orBookSize.width) {
				this._orBookSize.width = bookWidth;
			}
			if(!this._orBookSize.height) {
				this._orBookSize.height = bookHtml.height();
			}
			var newBookHeight = (adjustByWidth / this._orBookSize.width) * this._orBookSize.height;
			//console.log(newBookHeight, this._orBookSize.height);
			bookHtml.turn('size', adjustByWidth, newBookHeight);
			bookSizeUpdated = true;
		} else if(this._orBookSize.width) {
			if(this._orBookSize.width > adjustByWidth) {
				var newBookHeight = (adjustByWidth / this._orBookSize.width) * this._orBookSize.height;
				bookHtml.turn('size', adjustByWidth, newBookHeight);
			} else {
				bookHtml.turn('size', this._orBookSize.width, this._orBookSize.height);
				this._orBookSize.width = 0;
			}
			bookSizeUpdated = true;
		}
		if(bookSizeUpdated) {
			this.center();
			if(!stageSizeUpdated) {
				this._setOriginalSizes(stageWidth, stageHeight, shellHeight);
				kWidth = (adjustByWidth / this._orBookSize.width);
				newStageHeight = kWidth * this._orStageSize.height + (toolbarHeight / kWidth);
				newShellHeight = kWidth * this._orShellSize.height/* + (toolbarHeight / kWidth)*/;
				bookStage.height( newStageHeight );
				bookShell.height( newShellHeight );
			}
			this._toolbarSwitchResponsive(toolbar, adjustByWidth);
		}
	//}
};
drpBook.prototype._toolbarSwitchResponsive = function(toolbar, adjustByWidth) {
	if(this._bookData.toolbar && this._bookData.toolbar.switchResponsive) {
		var manipulateBtns = ['.dprTbCenter .dprTbBtnFirst', '.dprTbCenter .dprTbBtnLast', '.dprTdPageNumForm'];
		if(adjustByWidth <= this._bookData.toolbar.switchResponsive && !this._toolbarInResponsive) {
			toolbar.find( manipulateBtns.join(',') ).hide();
			toolbar.addClass('dprTbResponsiveOn');
			this._toolbarInResponsive = true;
		} else if(adjustByWidth > this._bookData.toolbar.switchResponsive && this._toolbarInResponsive) {
			toolbar.find( manipulateBtns.join(',') ).show();
			toolbar.removeClass('dprTbResponsiveOn');
			this._toolbarInResponsive = false;
		}
	}
};
drpBook.prototype._setOriginalSizes = function(stageWidth, stageHeight, shellHeight) {
	if(!this._orStageSize.width)
		this._orStageSize.width = stageWidth;
	if(!this._orStageSize.height)
		this._orStageSize.height = stageHeight;
	if(!this._orShellSize.height)
		this._orShellSize.height = shellHeight;
};
drpBook.prototype._adjustPosForZommed = function() {
	var bookHtml = this.getHtml()
	,	displayPagesNum = this._getDisplayPagesNum()
	,	bookWidth = this.get('prevWidth');

	if(displayPagesNum && displayPagesNum.length == 1) {
		if(displayPagesNum[0] == 1)	// For first page - move book to left side
			bookHtml.css('left', -((bookWidth / 2) * this._zoomedLevel)+ 'px');
		else	// For last page - decrease width to hide outside background
			bookHtml.css('width', (bookWidth / 2)+ 'px');
	} else {
		bookHtml.css({
			'left': this.get('prevLeft')
		,	'width': this.get('prevWidth')
		});
	}
};
drpBook.prototype.turnNext = function() {
	var bookHtml = this.getHtml()
	,	bookData = this.getData()
	,	currentPage = bookHtml.turn('page');
	if(currentPage < bookData.pages.length) {
		bookHtml.turn('next');
		if(this._zoomEnabled) {
			this._imgSetZoomSizes();
		}
	}
};
drpBook.prototype.turnPrev = function() {
	var bookHtml = this.getHtml()
	,	currentPage = bookHtml.turn('page');
	if(currentPage > 1) {
		bookHtml.turn('previous');
		if(this._zoomEnabled) {
			this._imgSetZoomSizes();
		}
	}
};
drpBook.prototype.getSocLinks = function() {
	return this.getToolbar().find('.dprTbSlShell');
};
drpBook.prototype.toggleSocLinks = function(newValue) {
	if(typeof(newValue) !== 'undefined') {
		newValue 
			? this.getSocLinks().show( this._anim ) 
			: this.getSocLinks().hide( this._anim );
	} else
		this.getSocLinks().toggle( this._anim );
};
drpBook.prototype._removeFakePages = function() {
	if(!this._bookData.params['fake_pages']) return;
	var bookShell = this.getBookShell();
	bookShell.find('.dprFakePage').remove();
};
drpBook.prototype._createFakePages = function() {
	if(!this._bookData.params['fake_pages']) return;
	var bookHtml = this.getHtml();
	if(!bookHtml) return;	// This mean book was not created
	var bookShell = this.getBookShell()
	,	currentPage = bookHtml.turn('page')
	,	fakeI = currentPage - 1
	,	bookWidth = bookHtml.width()
	,	bookHeight = bookHtml.height()
	,	shellWidth = bookShell.width()
	,	shellHeight = bookShell.height()
	,	fakeTop = (shellHeight - bookHeight) / 2
	,	fakeLeft = ((shellWidth - bookWidth) / 2) + bookWidth - 1	// 1 - for border
	,	i = 0
	,	fakeWidth = 4
	,	pageNums = this._getDisplayPagesNum()
	,	pagesPerFake = 10
	,	pagesCount = this._pagesCount;
	this._removeFakePages();
	if(pageNums.length == 1) {
		if(pageNums[0] != 1)	// If we see 1 page - then this is begining or end, if 1 - begining, else - end, and we don't need fake pages here
			return;
		fakeLeft -= bookWidth/4;
	}
	
	//this._pagesCount = 900;
	bookHeight -= 3;
	fakeLeft = Math.floor(fakeLeft);
	fakeTop = Math.floor(fakeTop);
	bookHeight = Math.floor(bookHeight);
	do {
		var clickTurnTo = fakeI + 1 + pagesPerFake
		,	newFakePage = jQuery('<div class="dprFakePage" data-page="'+ clickTurnTo+ '"/>')
		,	newFakePageCss = {
				'top': fakeTop
			,	'height': bookHeight+ 'px'
		};
		if(this._bookData.params['direction'] == 'rtl')
			newFakePageCss.right = fakeLeft;
		else
			newFakePageCss.left = fakeLeft

		newFakePage.css( newFakePageCss ).addClass(this._bookData.params['direction']);
		newFakePage.click(function(){
			var turnToPage = parseInt(jQuery(this).data('page'));
			if(turnToPage && bookHtml) {
				if(turnToPage > pagesCount)
					turnToPage = pagesCount;
				
				setTimeout(function(){
					bookHtml.turn('page', turnToPage);
				}, 100);
				
			}

		});
		bookShell.append(newFakePage);
		fakeLeft += fakeWidth;
		fakeTop += 2;
		bookHeight -= 4;
		fakeI += pagesPerFake;	// one fake peg per each 10 original pages
		i++;
	} while(fakeI < pagesCount);
};
drpBook.prototype.getExportLinks = function() {
	return this.getToolbar().find('.dprTbExportShell');
};
drpBook.prototype.toggleExportLinks = function(newValue) {
	if(typeof(newValue) !== 'undefined') {
		newValue 
			? this.getExportLinks().show( this._anim ) 
			: this.getExportLinks().hide( this._anim );
	} else
		this.getExportLinks().toggle( this._anim );
};
drpBook.prototype._checkBorderShadow = function() {
	if(!this._bookData.params['show_border_shadow']) return;
	var pageNums = this._getDisplayPagesNum()
	,	bookHtml = this.getHtml();
	if(!bookHtml) return;	// This mean book was not created
	if(pageNums.length == 1) {
		bookHtml.find('.page-wrapper:visible').addClass('borderShadowed');
	} else {
		bookHtml.addClass('borderShadowed');
	}
};
drpBook.prototype._removeBorderShadow = function() {
	if(!this._bookData.params['show_border_shadow']) return;
	var bookHtml = this.getHtml();
	bookHtml
		.removeClass('borderShadowed')
		.find('.page-wrapper')
		.removeClass('borderShadowed');
};
