<?php
class bookDpr extends moduleDpr {
	private $_assetsLoaded = false;
	private $_bookParamsDesc = array();
	public function init() {
		parent::init();
		add_shortcode('book_ready', array($this, 'displayBook'));
		add_action('wp_footer', array($this, 'addBooksDataToJs'));
		// Add same input fields to add Book form
		add_action(DPR_BOOK. '_add_form_fields', array($this, 'getBookAdminAddFormInput'));
		add_action(DPR_BOOK. '_edit_form_fields', array($this, 'getBookAdminFormInput'), 10, 2);
		// Add preview button at the bottom of edit form
		add_action(DPR_BOOK. '_edit_form', array($this, 'getBookAdminEditFormButtons'), 99, 2);
		// Add some book tabs at the bottom of the edit page
		add_action(DPR_BOOK. '_edit_form', array($this, 'getBookAdminEditTabs'), 10, 2);
		// 2 filters - one for save edited tax, other - for add new one form
		add_action('edited_'. DPR_BOOK, array($this, 'saveBookFormInput'), 10, 2);
		add_action('create_'. DPR_BOOK, array($this, 'saveBookFormInput'), 10, 2);
		// Add additional columns for Books
		add_filter('manage_edit-'. DPR_BOOK. '_columns', array($this, 'initBookAdminColumns'));
		add_filter('manage_'. DPR_BOOK. '_custom_column', array($this, 'addBookAdminColumns'), 10, 3);
		// Open our menu for Books menu item
		add_filter('parent_file', array($this, 'checkBooksAdminParentPage'));
		// Add our actions to admin books table - modify preview, "Add Page"
		add_filter(DPR_BOOK. '_row_actions', array($this, 'rebuildAdminTblActions'), 10, 2);
		// Add live preview to edit book form
		//add_action(DPR_BOOK. '_edit_form', array($this, 'getBookLivePreview'), 10, 2);
	}
	public function displayBook($params, $opts = array()) {
		$params['id'] = isset($params['id']) ? (int) $params['id'] : 0;
		if($params['id']) {
			$this->loadAssets();
			return  $this->getView()->getBookHtml($params);
		}
		return false;
	}
	public function loadAssets() {
		if(!$this->_assetsLoaded) {
			frameDpr::_()->addStyle('book', $this->getModPath(). 'css/book.css');
			frameDpr::_()->getModule('templates')->loadCoreJs();
			frameDpr::_()->addScript('modernizr', $this->getModPath(). 'js/modernizr.2.5.3.min.js');
			frameDpr::_()->addScript('book.class', $this->getModPath(). 'js/book.class.js');
			frameDpr::_()->addScript('book.manager.class', $this->getModPath(). 'js/book.manager.class.js');
			frameDpr::_()->addScript('book', $this->getModPath(). 'js/book.js', false, false, true);
			//frameDpr::_()->addJSVar('book', 'bookModPath', $this->getModPath());
			frameDpr::_()->addJSVar('book', 'bookCoreFolderPath', dispatcherDpr::applyFilters('bookCorePath', ''));
			$this->_assetsLoaded = true;
		}
	}
	public function loadAdminAssets() {
		frameDpr::_()->addScript('admin.book', $this->getModPath(). 'js/admin.book.js');
		frameDpr::_()->addJSVar('admin.book', 'dprDefaultAudioPaging', $this->getDefaultAudioPaging());
		frameDpr::_()->addStyle('admin.book', $this->getModPath(). 'css/admin.book.css');
	}
	public function addBooksDataToJs() {
		$books = $this->getView()->getUsedBooks();
		if(!empty($books)) {
			frameDpr::_()->addJSVar('book', 'dprAllBooks', $books);
		}
	}
	public function getBookAdminFormInput($tag) {
		$this->addLivePreviewAssets();
		frameDpr::_()->addScript('admin.book.edit', $this->getModPath(). 'js/admin.book.edit.js', false, true);
		$this->getView()->getBookAdminFormInput($tag);
	}
	public function getBookAdminAddFormInput($tag) {
		$this->getView()->getBookAdminFormInput($tag, array('addForm' => true));
	}
	public function getBookParamsDesc() {
		if(!$this->_bookParamsDesc) {
			$this->_bookParamsDesc = dispatcherDpr::applyFilters('bookParamsDesc', array(
				'stage_width' => array('label' => __('Stage Width', DPR_LANG_CODE), 'type' => 'text', 'default' => frameDpr::_()->getModule('options')->get('def_stage_width'), 'desc' => __('Defines the width of the publication background area in pixels or percents (%).', DPR_LANG_CODE), 'show_on_create' => true),
				'stage_height' => array('label' => __('Stage Height', DPR_LANG_CODE), 'type' => 'text', 'default' => frameDpr::_()->getModule('options')->get('def_stage_height'), 'desc' => __('Defines the height of the publication background area in pixels.', DPR_LANG_CODE), 'show_on_create' => true),
				'width' => array('label' => __('Width', DPR_LANG_CODE), 'type' => 'text', 'default' => frameDpr::_()->getModule('options')->get('def_width'), 'desc' => __('Defines the publication width in pixels.', DPR_LANG_CODE), 'show_on_create' => true),
				'height' => array('label' => __('Height', DPR_LANG_CODE), 'type' => 'text', 'default' => frameDpr::_()->getModule('options')->get('def_height'), 'desc' => __('Defines the publication height in pixels.', DPR_LANG_CODE), 'show_on_create' => true),
				
				'bg_type' => array('label' => __('Publication Background', DPR_LANG_CODE), 'type' => 'selectbox', 'default' => '', 'params' => array('options' => array('' => __('None'), 'color' => __('Color'), 'img' => __('Image', DPR_LANG_CODE)))),
				'bg_color' => array('label' => __('Background Color', DPR_LANG_CODE), 'type' => 'colorpicker', 'default' => '#ffffff', DPR_LANG_CODE),
				'bg_opacity' => array('label' => __('Background Opacity', DPR_LANG_CODE), 'type' => 'slider', 'default' => 1, 'params' => array('min' => 0, 'max' => 1, 'step' => 0.1, 'slide' => 'dprOnAdminOpacitySlideMove', DPR_LANG_CODE)),
				'bg_img' => array('label' => __('Background Image', DPR_LANG_CODE), 'type' => 'imgGalleryBtn', 'params' => array('onChange' => 'dprOnAdminBgImgChange', 'attrs' => 'class="button"', DPR_LANG_CODE)),
				'bg_img_pos' => array('label' => __('Background Image Position', DPR_LANG_CODE), 'type' => 'selectbox', 'params' => array('options' => array('center' => __('Center'), 'tile' => __('Tile'), 'stretch' => __('Stretch', DPR_LANG_CODE)))),
				
				'page_bg_type' => array('label' => __('Page Background', DPR_LANG_CODE), 'type' => 'selectbox', 'default' => 'color', 'params' => array('options' => array('' => __('None', DPR_LANG_CODE), 'color' => __('Color', DPR_LANG_CODE), 'img' => __('Image', DPR_LANG_CODE)))),
				'page_bg_color' => array('label' => __('Page Background Color', DPR_LANG_CODE), 'type' => 'colorpicker', 'default' => '#ffffff', DPR_LANG_CODE),
				'page_bg_opacity' => array('label' => __('Page Background Opacity', DPR_LANG_CODE), 'type' => 'slider', 'default' => 1, 'params' => array('min' => 0, 'max' => 1, 'step' => 0.1, 'slide' => 'dprOnAdminOpacitySlideMove', DPR_LANG_CODE)),
				'page_bg_img' => array('label' => __('Page Background Image', DPR_LANG_CODE), 'type' => 'imgGalleryBtn', 'params' => array('onChange' => 'dprOnAdminPageBgImgChange', 'attrs' => 'class="button"', DPR_LANG_CODE)),
				'page_bg_img_pos' => array('label' => __('Page Background Image Position', DPR_LANG_CODE), 'type' => 'selectbox', 'params' => array('options' => array('center' => __('Center'), 'tile' => __('Tile'), 'stretch' => __('Stretch', DPR_LANG_CODE)))),
				
				'open_on_page' => array('label' => __('Open Publication on page', DPR_LANG_CODE), 'type' => 'text', 'desc' => __('Will open publication on provided page number, empty or 0 - will open closed publication', DPR_LANG_CODE)),
				'toolbar_code' => array('label' => __('Toolbar', DPR_LANG_CODE), 'not_null' => 'simple', 'type' => 'hidden'),
				'enable_pages_dynamic_preload' => array('label' => __('Pre-load pages one-by-one', DPR_LANG_CODE), 'type' => 'checkboxHiddenVal', 'default' => '1', 'desc' => __('Pages content will not be loaded all when publication load, it will be pre-loaded when page is required to show', DPR_LANG_CODE)),
				'audio_paging' => array('label' => __('Flip Sound', DPR_LANG_CODE), 'type' => 'audioGalleryBtn', 'default' => $this->getDefaultAudioPaging(), 'desc' => __('Select audio file, this feature is supported only in HTML5 browsers'), 'params' => array('onChange' => 'dprOnAdminAudioChange', 'attrs' => 'class="button"', DPR_LANG_CODE)),
				'en_audio_paging' => array('label' => __('Enable audio paging', DPR_LANG_CODE), 'type' => 'checkboxHiddenVal', 'default' => '1', DPR_LANG_CODE),
				'direction' => array('label' => __('Listing direction', DPR_LANG_CODE), 'type' => 'selectbox', 'desc' => __('Publication pages listing direction'), 'default' => 'ltr', 'params' => array('options' => array('ltr' => __('Left to Right'), 'rtl' => __('Right to Left', DPR_LANG_CODE)))),
				'cover_type' => array('label' => __('Cover type', DPR_LANG_CODE), 'type' => 'selectbox', 'default' => 'softcover', 'params' => array('options' => array('softcover' => __('Softcover'), 'hardcover' => __('Hardcover', DPR_LANG_CODE)))),
				'duration' => array('label' => __('Transition duration', DPR_LANG_CODE), 'type' => 'text', 'default' => '600', 'desc' => __('Sets the duration of the transition in milliseconds. If you set 0 (zero), there won\'t be transition while turning the page.', DPR_LANG_CODE)),
				'gradients' => array('label' => __('Gradients', DPR_LANG_CODE), 'type' => 'checkboxHiddenVal', 'default' => '1', 'desc' => __('Shows gradients and shadows during the transition.', DPR_LANG_CODE)),
				'page_content_indent' => array('label' => __('Page content indent', DPR_LANG_CODE), 'type' => 'text', 'default' => '0', 'desc' => __('Indent from edge of the page to it\'s internal content in px', DPR_LANG_CODE)),
				
				'social_links' => array('label' => __('Social links', DPR_LANG_CODE), 'type' => 'hidden', 'default' => '', 'desc' => __('Social links for your publication', DPR_LANG_CODE)),
				'fake_pages' => array('label' => __('Show volume fake pages', DPR_LANG_CODE), 'type' => 'checkboxHiddenVal', 'default' => '0', 'desc' => __('Enable effect with volume page - like in real publication.', DPR_LANG_CODE)),
				
				'show_export' => array('label' => __('Show export buttons', DPR_LANG_CODE), 'type' => 'checkboxHiddenVal', 'default' => '1', 'desc' => __('Enable export publication button - to html or pdf.', DPR_LANG_CODE)),
				'zoom_level' => array('label' => __('Zoom level', DPR_LANG_CODE), 'type' => 'text', 'default' => '2', 'desc' => __('Zoom level, that will be set after you click on Zoom button', DPR_LANG_CODE)),
				
				'show_border_shadow' => array('label' => __('Show border shadow', DPR_LANG_CODE), 'type' => 'checkboxHiddenVal', 'default' => '1', 'desc' => __('Shows border shadow on the perimeter of the book.', DPR_LANG_CODE)),
			));
		}
		return $this->_bookParamsDesc;
	}
	public function saveBookFormInput($termId) {
		$data = reqDpr::getVar('dpr');
		if(empty($data)) {
			// This is used for example to update book options when add batch pages, @see bookControllerDpr::addPagesFromImages()
			$bookParams = reqDpr::getVar('bookParams');
			if(!empty($bookParams) && isset($bookParams['dpr']) && !empty($bookParams['dpr'])) {
				$data = $bookParams['dpr'];
			}
		}
		if(!empty($data)) {
			$data['id'] = $termId;
			$this->getModel()->saveBookParams($data);
			frameDpr::_()->getModule('promo_ready')->getModel()->saveUsageStat('book.save');
		}
	}
	public function initBookAdminColumns($cols) {
		$keys = array('cb', 'id', 'name', 'pages_list', 'shortcode', 'params');
		$sortedCols = array();
		$cols['id'] = __('ID', DPR_LANG_CODE);
		$cols['shortcode'] = __('Shortcode', DPR_LANG_CODE);
		$cols['params'] = __('Params', DPR_LANG_CODE);
		$cols['pages_list'] = __('Pages', DPR_LANG_CODE);
		unset($cols['description']);
		unset($cols['slug']);
		unset($cols['posts']);
		foreach($keys as $key) {
			$sortedCols[ $key ] = $cols[ $key ];
		}
		return $sortedCols;
	}
	public function addBookAdminColumns($out, $columnName, $termId) {
		switch ($columnName) {
			case 'id':
				$out = $termId;
				break;
			case 'pages_list':
				$book = $this->getModel()->getById($termId, false, true);
				if($book->count) {
					$out = '<a title="'. __('View publication pages list').'" href="'. admin_url('edit.php?post_type='. DPR_PAGE. '&'. DPR_BOOK. '='. $book->slug). '" target="_blank">'. __('Total pages', DPR_LANG_CODE). ': '. $book->count. '</a>';
					$out .= '<br />';
					$out .= '<a title="'. __('Edit publication pages'). '" href="'. $this->getLinkBookEdit($termId, array('hash' => 'dprAdminBookPages')). '">'. __('Edit', DPR_LANG_CODE). '</a>';
				} else {
					$out = '<a href="'. $this->getLinkAddPageForBook($termId). '">'. __('Start add Pages', DPR_LANG_CODE). '</a>';
				}
				break;
			case 'shortcode':
				$out = $this->generateShortcodeHtml($termId);
				break;
			case 'params':
				$paramsDesc = $this->getBookParamsDesc();
				$params = $this->getModel()->getBookParams($termId);
				$out = $this->_paramsToTable($paramsDesc, $params);
				break;
		}
		return $out;
	}
	public  function generateShortcode($bookId) {
		return '[book_ready id="'. $bookId. '"]';
	}
	public function generateShortcodeHtml($bookId) {
		return '<div class="gmpShortCodePreview"><span>'. $this->generateShortcode($bookId). '</span></div>';
	}
	/**
	 * Just unused for now - until I will image how it will correct and pretty display
	 */
	private function _paramsToTable($paramsDesc, $params) {
		$showOnlyKeys = array('stage_width', 'stage_height', 'width', 'height');
		$res = '';
		$res .= '<table class="dprAdminBookParamsShowListTbl">';
		foreach($showOnlyKeys as $key) {
			$res .= '<tr><td>'. $paramsDesc[ $key ]['label']. '</td><td>'. (isset($params[$key]) ? $params[$key] : ''). '</td></tr>';
		}
		$res .= '</table>';
		return $res;
	}
	public function checkBooksAdminParentPage($parent_file) {
		if(frameDpr::_()->getModule('pages')->isAdminBooksPage()) {
			$parent_file = frameDpr::_()->getModule('adminmenu')->getMainSlug();
		}
		return $parent_file;
	}
	public function rebuildAdminTblActions($actions, $tag) {
		$actions['view'] = '<a target="_blank" href="'. $this->getLinkPreviewBook(array('id' => $tag->term_id)) .'">'. __('Preview', DPR_LANG_CODE). '</a>';
		$actions['add_page'] = '<a href="'. $this->getLinkAddPageForBook($tag->term_id). '">'. __('Add Page', DPR_LANG_CODE). '</a>';
		return $actions;
	}
	public function getLinkAddPageForBook($termId) {
		return admin_url('post-new.php?post_type='. DPR_PAGE. '&'. DPR_BOOK. '='. $termId);
	}
	public function getLinkPreviewBook($params) {
		$url = uriDpr::mod('book', 'preview', array('id' => $params['id']));
		if(isset($params['page']) && $params['page']) {
			$url .= '#page'. $params['pageId'];
		}
		return $url;
	}
	public function getLinkPageEdit($pageId) {
		return admin_url('post.php?post='. $pageId. '&action=edit');
	}
	public function getLinkBookEdit($bookId, $params = array()) {
		$url = admin_url('edit-tags.php?action=edit&taxonomy='. DPR_BOOK. '&tag_ID='. $bookId. '&post_type=post');
		if(isset($params['hash']) && $params['hash']) {
			$url .= '#'. $params['hash'];
		}
		return $url;
	}
	public function getLinkBookAdd() {
		return admin_url('edit-tags.php?taxonomy='. DPR_BOOK);
	}
	public function addLivePreviewAssets() {
		frameDpr::_()->addScript('admin.book.livepreview', $this->getModPath(). 'js/admin.book.livepreview.js');
	}
	public function getBookAdminEditFormButtons($tag, $taxonomy) {
		if($taxonomy == DPR_BOOK) {	// Additional check
			$this->getView()->displayEditBookFormButtons($tag);
		}
	}
	public function getBookAdminEditTabs($tag, $taxonomy) {
		if($taxonomy == DPR_BOOK) {	// Additional check
			frameDpr::_()->addScript('wp.tabs', DPR_JS_PATH. 'wp.tabs.js');
			frameDpr::_()->addScript('jquery-ui-sortable');
			$this->getView()->displayEditBookFormTabs($tag);
		}
	}
	public function getDefaultAudioPaging() {
		static $url = '';
		if(empty($url)) {
			$url = $this->getModPath(). 'files/page-turn.wav';
		}
		return $url;
	}
	public function canUsePdfExport() {
		return version_compare(phpversion(), '5.3', '>=');
	}
}
