<?php
class bookModelDpr extends modelDpr {
	private $_batchResults = array();
	public function getById($id, $withPages = true, $light = false, $by = 'id') {
		$book = get_term_by($by, $id, DPR_BOOK);
		if($light)
			return $book;
		if($book) {
			$book = $this->_afterGet($book, $withPages);
			return $book;
		} else
			$this->pushError (__('Can not find book by ID', DPR_LANG_CODE));
		return false;
	}
	private function _afterGet($book, $withPages = true) {
		if($withPages) {
			$book->pages = frameDpr::_()->getModule('page')->getModel()->getListForBookByBookId($book->term_id);
		}
		$book->params = $this->getBookParams($book->term_id);
		return $book;
	}
	public function getBookParams($id) {
		$params = get_option(DPR_BOOK. '_params_'. $id);
		if(!$params) {
			$params = array();
		}
		$params = $this->preFillDefParams($params);
		return $params;
	}
	public function preFillDefParams($params) {
		$paramsDesc = $this->getModule()->getBookParamsDesc();
		foreach($paramsDesc as $key => $data) {
			if(!isset($params[ $key ]) && isset($data['default'])) {
				$params[ $key ] = $data['default'];
			}
			if(empty($params[ $key ]) && isset($data['not_null'])) {
				$params[ $key ] = $data['not_null'];
			}
		}
		return $params;
	}
	public function saveBookParams($d = array()) {
		if($d['id']) {
			$id = $d['id'];
			$paramsDesc = $this->getModule()->getBookParamsDesc();
			$params = $this->getBookParams($id);
			foreach($paramsDesc as $key => $data) {
				if(isset($d[ $key ])) {
					$params[ $key ] = $d[ $key ];
				} elseif(isset($data['default'])) {
					$params[ $key ] = $data['default'];
				}
			}
			update_option(DPR_BOOK. '_params_'. $id, $params);
			if(isset($d['pages_sort_order'])) {
				$this->updatePagesSortOrder($id, $d['pages_sort_order']);
			}
			return $id;
		}
		return false;
	}
	public function updatePagesSortOrder($id, $pagesSortOrder) {
		if(is_string($pagesSortOrder)) {
			$pagesSortOrderArr = explode(',', $pagesSortOrder);
			$pagesSortOrder = array();
			foreach($pagesSortOrderArr as $pageSort) {
				$pageSortArr = array_map('trim', explode(':', $pageSort));
				$pagesSortOrder[ $pageSortArr[0] ] = $pageSortArr[1];
			}
		}
		$this->updateSortOrderFromArray($id, $pagesSortOrder);
	}
	public function updateSortOrderFromArray($id, $pagesSortOrderArr) {
		frameDpr::_()->getTable('page_sort_order')->delete(array('book_id' => $id));
		if(!empty($pagesSortOrderArr)) {
			$valuesArr = array();
			foreach($pagesSortOrderArr as $pageId => $sortOrder) {
				$valuesArr[] = '('. $id. ', '. $pageId. ', '. $sortOrder. ')';
			}
			$finalInsertQuery = 'INSERT INTO @__page_sort_order (book_id, page_id, sort_order) VALUES '. implode(',', $valuesArr);
			dbDpr::query($finalInsertQuery);
		}
	}
	public function getPagesSortOrder($id) {
		return frameDpr::_()->getTable('page_sort_order')
				->orderBy('sort_order ASC')
				->get('*', array('book_id' => $id));
	}
	public function getPagesSortOrderByPageId($id) {
		$res = array();
		$sortOrder = $this->getPagesSortOrder($id);
		if($sortOrder) {
			foreach($sortOrder as $so) {
				$res[ $so['page_id'] ] = $so['sort_order'];
			}
		}
		return $res;
	}
	public function getList($d, $withPages = true) {
		$books = get_terms(DPR_BOOK, $d);
		if(!empty($books)) {
			foreach($books as $i => $book) {
				$books[ $i ] = $this->_afterGet($books[ $i ], $withPages);
			}
		}
		return $books;
	}
	public function addPagesFromImages($d = array()) {
		$id = isset($d['id']) ? (int) $d['id'] : 0;
		if($id) {
			if(isset($d['batch_pages']) && !empty($d['batch_pages'])) {
				$d = dispatcherDpr::applyFilters('beforeAddPagesFromImages', $d);
				$pagesCount = count($d['batch_pages']['titles']);
				$this->_initBatchRes($pagesCount);
				for($i = ($pagesCount - 1); $i >= 0; $i--) {
					$addPageParams = array(
						'title' => $d['batch_pages']['titles'][ $i ],
						'book_id' => $id,
					);
					if(isset($d['batch_pages']['attach_id']) && $d['batch_pages']['attach_id'][ $i ]) {
						$addPageParams['attach_id'] = $d['batch_pages']['attach_id'][ $i ];
					} elseif(isset($d['batch_pages']['img_url']) && $d['batch_pages']['img_url'][ $i ]) {
						$addPageParams['img_url'] = $d['batch_pages']['img_url'][ $i ];
					}
					$newPageId = frameDpr::_()->getModule('page')->getModel()->addPageFromImg( $addPageParams );
					$newPageId ? $this->_addBatchSuccess() : $this->_addBatchFail();
				}
				return true;
			} else
				$this->pushError (__('Empty pages data', DPR_LANG_CODE));
		} else
			$this->pushError (__('Empty or invalid ID', DPR_LANG_CODE));
		return false;
	}
	private function _addBatchSuccess() {
		$this->_initBatchRes();
		$this->_batchResults['success']++;
	}
	private function _addBatchFail() {
		$this->_initBatchRes();
		$this->_batchResults['fail']++;
	}
	private function _initBatchRes($resetTotal = false) {
		if(empty($this->_batchResults) || $resetTotal) {
			$this->_batchResults = array(
				'success' => 0,
				'fail' => 0,
				'total' => $resetTotal,
			);
		}
	}
	public function getBatchResults() {
		return $this->_batchResults;
	}
	public function parseFolderForImages($d = array()) {
		$folder = isset($d['folder']) ? trim($d['folder']) : '';
		if(!empty($folder)) {
			$wpUploadDir = wp_upload_dir();
			// Remove all that is before uploads folder - we will add correct filesystem path after this condition
			if($uplodDirEntr = strpos($folder, 'wp-content/uploads') !== false) {
				$folder = preg_replace('/.*wp-content\/uploads\/*/i', '', $folder);
			}
			if(strpos($folder, 'wp-content/uploads') === false) {
				// Remove directory separator from begining if it is there - we will add it after as DS
				if(strpos($folder, '/') === 0) {
					$folder = substr($folder, 1);
				}
				$folder = $wpUploadDir['basedir']. DS. $folder;
			}
			$files = utilsDpr::getFilesList($folder);
			if(!empty($files)) {
				$returnFilesUrls = array();
				$dirRelativePath = str_replace(DS, '/',
						str_replace(array('https://', 'http://'), '',
						str_replace('wp-content/uploads', '', 
						str_replace($wpUploadDir['basedir'], '', $folder))));
				if(strpos($dirRelativePath, '/') !== 0)
						$dirRelativePath = '/'. $dirRelativePath;
				if(strpos($dirRelativePath, '/') !== strlen($dirRelativePath))
						$dirRelativePath = $dirRelativePath. '/';
				foreach($files as $file) {
					$filetype = wp_check_filetype( basename( $file ), null );
					if($filetype && strpos($filetype['type'], 'image') !== false) {
						$returnFilesUrls[] = array(
							'path' => $wpUploadDir['baseurl']. $dirRelativePath. $file,
							'title' => preg_replace('/\..{2,4}$/i', '', $file),
						);
					}
				}
				if(!empty($returnFilesUrls)) {
					return $returnFilesUrls;
				} else
					$this->pushError (__('Can not find images in provided folder path', DPR_LANG_CODE));
			} else
				$this->pushError (__('Can not open directory, or directory is empty', DPR_LANG_CODE));
		} else 
			$this->pushError (__('Empty folder path'), 'add_batch_serv_path', DPR_LANG_CODE);
		return false;
	}
	public function getTotalCount() {
		return wp_count_terms(DPR_BOOK);
	}
}