<?php
class bookViewDpr extends viewDpr {
	private $_usedBooks = array();
	private $_toolbarList = array();
	public function getBookHtml($params) {
		$book = $this->getModel()->getById($params['id']);
		if(!empty($book)) {
			$book = $this->prepareBookToOut($book);
			$bookParamsDesc = $this->getModule()->getBookParamsDesc();
			foreach($bookParamsDesc as $k => $v) {
				if(isset($params[ $k ])) {
					$book->params[ $k ] = $params[ $k ];
				}
			}
			$book = $this->_prepareBookPages($book);
			$book = dispatcherDpr::applyFilters('bookBeforeOut', $book);
			$this->_addUsedBook($book);
			$book->toolbar = $this->getToolbarDataByCode($book->params['toolbar_code']);
			if(isset($book->toolbar['css'])) {
				frameDpr::_()->addStyle($book->params['toolbar_code'], $book->toolbar['css']);
			}
			$this->assign('book', $book);
			$this->assign('toolbarHtml', $this->getToolbarHtml($book));
			$this->assign('toolbarData', $book->toolbar);
			return parent::getInlineContent('bookDisplay');
		} else {
			$modelErrors = frameDpr::_()->getModule('page')->getModel()->getErrors();
			if(empty($modelErrors)) {
				$modelErrors = $this->getModel()->getErrors();
			}
			if(empty($modelErrors))
				$modelErrors = __('Error display book', DPR_LANG_CODE);
			$this->assign('modelErrors', $modelErrors);
			return parent::getContent('bookDisplayErrors');
		}
	}
	private function _preparePageImgs($book, $page, $attachIds, $forceUseWpThumb = false) {
		$attachModel = $this->getModel('attach');
		$width = isset($book->params) 
			? (isset($book->params['width_out']) ? (int) $book->params['width_out'] : (int) $book->params['width']) 
			: 0;
		$height = isset($book->params) 
			? (isset($book->params['height_out']) ? (int) $book->params['height_out'] : (int) $book->params['height']) 
			: 0;
		$attachId = $attachIds[0];
		if($width && $height) {
			$width /= 2;	// As one page - is have full width/2 width
			$resizedUrl = $attachModel->getAttachment($attachId, $width, $height, false, $book->term_id);
		} else if($forceUseWpThumb) {
			$resizedUrl = wp_get_attachment_image_src($attachId, 'thumbnail');
			if($resizedUrl && $resizedUrl[0]) {
				$resizedUrl = $resizedUrl[0];
			}
		}
		if($resizedUrl && !strpos($page->post_content, $resizedUrl)) {
			preg_match('/src="([^"]*)"/i', $page->post_content, $imgSrcRes);
			if($imgSrcRes && $imgSrcRes[1]) {
				$page->post_content = str_replace($imgSrcRes[1], $resizedUrl, $page->post_content);
			}
		}
		return $page;
	}
	private function _prepareBookPages($book, $params = array()) {
		//$start = microtime(true);
		global $post;
		$needRedefinePost = (!empty($post) && is_object($post));
		if($needRedefinePost) {
			$tmpPost = clone($post);
		}
		if(!empty($book->pages)) {
			foreach($book->pages as $i => $page) {
				$attachIds = utilsDpr::parseImgIds($page->post_content);
				$page->attach_sizes = array();
				if(!empty($attachIds)) {
					if($page->attach_id) {
						$forseUseWpThumb = isset($params['forceUseWpThumb']) && $params['forceUseWpThumb'] ? $params['forceUseWpThumb'] : false;
						$page = $this->_preparePageImgs($book, $page, $attachIds, $forseUseWpThumb);
					}
					foreach($attachIds as $aid) {
						if(!isset($page->attach_sizes[ $aid ])) {
							$fullImgData = wp_get_attachment_image_src($aid, 'full');
							$page->attach_sizes[ $aid ] = array('full' => $fullImgData[0]);
						}
					}
				}
				// We need this to prevent unwanted the_content hooks
				if($needRedefinePost) {
					$post = $page;
				}
				$page->post_content = apply_filters('the_content', $page->post_content);
				$page->post_content = str_replace(']]>', ']]&gt;', $page->post_content);
				if(isset($params['includeEditLink']) && $params['includeEditLink']) {
					$page->edit_link = $this->getModule()->getLinkPageEdit($page->ID);
				}
			}
		}
		if($needRedefinePost) {
			$post = $tmpPost;
		}
		//echo '<br /><b>'. (microtime(true) - $start). '</b>';
		return $book;
	}
	private function _addUsedBook($book) {
		$this->_usedBooks[] = $book;
	}
	public function getUsedBooks() {
		return $this->_usedBooks;
	}
	public function getBookAdminFormInput($book, $params = array()) {
		frameDpr::_()->getModule('templates')->loadCoreJs();
		frameDpr::_()->getModule('templates')->loadCoreCss();
		$isAddForm = (isset($params['addForm']) && $params['addForm']);
		if($book && is_object($book))
			$bookParams = $this->getModel()->getBookParams($book->term_id);
		else
			$bookParams = array();
		if(!$isAddForm) {
			$this->assign('toolbars', $this->getToolbarsList());
		}
		$this->assign('bookParams', $bookParams);
		$this->assign('bookParamsDesc', $this->getModule()->getBookParamsDesc());
		$this->assign('isAddForm', $isAddForm);
		return parent::display('bookAdminFormInput');
	}
	private function _getExportLinks($book) {
		$exportLinks = array(
			'html' => array('label' => __('HTML', DPR_LANG_CODE), 'link' => uriDpr::mod('book', 'exportToHtml', array('id' => $book->term_id))),
		);
		if($this->getModule()->canUsePdfExport()) {
			$exportLinks['pdf'] = array('label' => __('PDF', DPR_LANG_CODE), 'link' => uriDpr::mod('book', 'exportToPdf', array('id' => $book->term_id)));
		}
		return $exportLinks;
	}
	/**
	 * @param string $code Toolbar code
	 */
	public function getToolbarHtml($book) {
		$socLinks = false;
		$exportLinks = false;
		if(isset($book->params['social_links']) && !empty($book->params['social_links'])) {
			$socLinks = frameDpr::_()->getModule('social_links')->prepareSocLinks($book->params['social_links']);
			if(!empty($socLinks)) {
				frameDpr::_()->getModule('social_links')->loadAssets();
			}
		}
		if(isset($book->params['show_export']) && !empty($book->params['show_export'])) {
			$exportLinks = $this->_getExportLinks($book);
		}
		$this->assign('book', $book);
		$this->assign('toolbarData', $this->getToolbarDataByCode($book->params['toolbar_code']));
		$this->assign('socLinks', $socLinks);
		$this->assign('exportLinks', $exportLinks);
		$toolbarTpl = 'bookToolbar_'. $book->params['toolbar_code'];
		return parent::getContent($toolbarTpl);
	}
	public function prepareBookToOut($book) {
		$prepareSizeKeys = array('stage_width', 'stage_height', 'width', 'height');
		foreach($prepareSizeKeys as $key) {
			$book->params[ $key. '_out' ] = $this->sizeToPxPt($book->params[ $key ]);
		}
		return $book;
	}
	
	public function getToolbarsList() {
		if(empty($this->_toolbarList)) {
			$this->_toolbarList = dispatcherDpr::applyFilters('bookToolbars', array(
				'simple' => array('height' => 40, 'css' => $this->getModule()->getModPath(). 'css/bookTb_simple.css', 'switchResponsive' => 435),
				'night' => array('height' => 40, 'css' => $this->getModule()->getModPath(). 'css/bookTb_night.css', 'switchResponsive' => 465),
			));
			foreach($this->_toolbarList as $code => $tb) {
				if(!isset($this->_toolbarList[ $code ]['prev_img'])) {
					$this->_toolbarList[ $code ]['prev_img'] = $this->getModule()->getModPath(). 'img/tb_previews/tb_'. $code. '.png';
				}
			}
		}
		return $this->_toolbarList;
	}
	public function getToolbarDataByCode($code) {
		$this->getToolbarsList();
		return isset($this->_toolbarList[$code]) ? $this->_toolbarList[$code] : $this->_toolbarList['simple'];	// $this->_toolbarList['simple'] - just for now, it should be false there
	}
	public function displayBookPreview($d = array()) {
		$this->assign('book', $this->getModel()->getById($d['id'], false));
		$this->assign('id', $d['id']);
		$this->assign('bookHtml', $d['bookHtml']);
		parent::display('bookPreview');
	}
	public function displayEditBookFormButtons($tag) {
		$this->assign('previewUrl', $this->getModule()->getLinkPreviewBook(array('id' => $tag->term_id)));
		$this->assign('tag', $tag);
		$this->assign('shortcodeHtml', $this->getModule()->generateShortcodeHtml($tag->term_id));
		parent::display('bookFormButtons');
	}
	public function displayEditBookFormTabs($tag) {
		$tag->params = $this->getModel()->getBookParams($tag->term_id);
		$tag->pages = frameDpr::_()->getModule('page')->getModel()->getListForBookByBookId($tag->term_id);
		$tag = $this->_prepareBookPages($tag, array('includeEditLink' => true, 'forceUseWpThumb' => true));
		frameDpr::_()->addJSVar('admin.book.edit', 'dprBook', (array) $tag);
		$this->assign('book', $tag);
		$this->assign('addBookPageUrl', $this->getModule()->getLinkAddPageForBook($tag->term_id));
		$this->assign('gdSupport', frameDpr::_()->getModule('dpr')->support('gd'));
		$this->assign('addTabs', dispatcherDpr::applyFilters('bookAdminAddTabs', array(), $tag));
		parent::display('bookFormTabs');
	}
	public function getExportHtml($id, $params = array()) {
		$book = $this->getModel()->getById($id);
		if(!empty($book)) {
			$book = $this->prepareBookToOut($book);
			$book = $this->_prepareBookPages($book);
			$this->assign('book', $book);
			$this->assign('simple', (isset($params['simple']) ? $params['simple'] : false));
			return parent::getInlineContent('bookExportHtml');
		}
	}
}
